#include "MainWindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindowClass)
{
    ui->setupUi(this);
}

void MainWindow::init() {

    int i=0;

    // initialize all variables
    for(i=0; i<27; i++) {
        ledsColor[i] = 0;
    }
    currentLed = 0;
    IRintensity = 64;
    snprintf(endCommand, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 145, 1);      // initialize end command string

    memset(buffer, 0x0, 100);
    sprintf(buffer, "\r");
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, strlen(buffer));
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, strlen(buffer), 100000);                // clear output buffer
    #endif

    memset(buffer, 0x0, 100);
    snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 0x90, 3);
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->WriteBuffer((BYTE*)endCommand, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);                             // 3-Colour LED Synchronous mode (3 colours changing in one cycle)
        comm->writeData(endCommand, 4, 100000);                         // send the end command to update the parameters
    #endif


}

void MainWindow::connect() {
    int err = 0;

    #ifdef __WIN32__
        comm = new TCommPort();
        comm->SetCommPort(ui->txtDevice->text().toLatin1().data());
        comm->SetBaudRate(115200);
        comm->SetParity(NOPARITY);      // NOPARITY and friends are #defined in windows.h
        comm->SetByteSize(8);
        comm->SetStopBits(ONESTOPBIT);  // ONESTOPBIT is also from windows.h
        err = comm->OpenCommPort();
    #else
        speed_t baud_rate = B115200;
        comm = new SerialComm();
        err = comm->connect(ui->txtDevice->text().toAscii().data(), baud_rate);
    #endif
    if(err == -1) {       
        perror("Cannot open serial port!");
        QMessageBox::critical( this, "RGB Panel Demo", "Unable to open serial port.\n");
        comm = NULL;
        return;
    }

    ui->btnConnect->setEnabled(false);

    init();

}

MainWindow::~MainWindow()
{
    if(comm != NULL) {
        turnLedsOff();
        #ifdef __WIN32__
            Sleep(50);
        #else
            usleep(50000);                     // wait turning off the leds
        #endif
        memset(buffer, 0x0, 100);
        snprintf(buffer, 2, "%c", 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 1);
            comm->FlushCommPort();
            comm->PurgeCommPort();
            comm->CloseCommPort();
        #else
            comm->writeData(buffer, 1, 100000); // end binary command
            comm->disconnect();
        #endif

    }
    delete ui;
}

void MainWindow::updateLed0() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[0]/128.0*255.0), int((double)ledsColor[1]/128.0*255.0), int((double)ledsColor[2]/128.0*255.0));
    ui->lblLed0->setStyleSheet(buffer2);
}

void MainWindow::updateLed1() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[3]/128.0*255.0), int((double)ledsColor[4]/128.0*255.0), int((double)ledsColor[5]/128.0*255.0));
    ui->lblLed1->setStyleSheet(buffer2);
}

void MainWindow::updateLed2() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[6]/128.0*255.0), int((double)ledsColor[7]/128.0*255.0), int((double)ledsColor[8]/128.0*255.0));
    ui->lblLed2->setStyleSheet(buffer2);
}

void MainWindow::updateLed3() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[9]/128.0*255.0), int((double)ledsColor[10]/128.0*255.0), int((double)ledsColor[11]/128.0*255.0));
    ui->lblLed3->setStyleSheet(buffer2);
}

void MainWindow::updateLed4() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[12]/128.0*255.0), int((double)ledsColor[13]/128.0*255.0), int((double)ledsColor[14]/128.0*255.0));
    ui->lblLed4->setStyleSheet(buffer2);
}

void MainWindow::updateLed5() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[15]/128.0*255.0), int((double)ledsColor[16]/128.0*255.0), int((double)ledsColor[17]/128.0*255.0));
    ui->lblLed5->setStyleSheet(buffer2);
}

void MainWindow::updateLed6() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[18]/128.0*255.0), int((double)ledsColor[19]/128.0*255.0), int((double)ledsColor[20]/128.0*255.0));
    ui->lblLed6->setStyleSheet(buffer2);
}

void MainWindow::updateLed7() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[21]/128.0*255.0), int((double)ledsColor[22]/128.0*255.0), int((double)ledsColor[23]/128.0*255.0));
    ui->lblLed7->setStyleSheet(buffer2);
}

void MainWindow::updateLed8() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", int((double)ledsColor[24]/128.0*255.0), int((double)ledsColor[25]/128.0*255.0), int((double)ledsColor[26]/128.0*255.0));
    ui->lblLed8->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR0Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR0->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR0Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR0->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR1Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR1->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR1Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR1->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR2Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR2->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR2Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR2->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR3Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR3->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR3Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR3->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR4Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR4->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR4Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR4->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR5Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR5->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR5Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR5->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR6Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR6->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR6Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR6->setStyleSheet(buffer2);
}

void MainWindow::turnOnIR7Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 255, 0, 0);
    ui->lblIR7->setStyleSheet(buffer2);
}

void MainWindow::turnOffIR7Slot() {
    memset(buffer2, 0x0, 100);
    sprintf(buffer2, "background-color: rgb(%d, %d, %d);", 0, 0, 0);
    ui->lblIR7->setStyleSheet(buffer2);
}

void MainWindow::updateR(int value) {

    memset(buffer, 0x0, 100);

    switch(currentLed) {
        case 0: ledsColor[0] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 0, value);
                updateLed0();
                break;
        case 1: ledsColor[3] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 1, value);
                updateLed1();
                break;
        case 2: ledsColor[6] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 2, value);
                updateLed2();
                break;
        case 3: ledsColor[9] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 3, value);
                updateLed3();
                break;
        case 4: ledsColor[12] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 4, value);
                updateLed4();
                break;
        case 5: ledsColor[15] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 5, value);
                updateLed5();
                break;
        case 6: ledsColor[18] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 6, value);
                updateLed6();
                break;
        case 7: ledsColor[21] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 7, value);
                updateLed7();
                break;
        case 8: ledsColor[24] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 146, value);
                updateLed8();
                break;
    }

    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->WriteBuffer((BYTE*)endCommand, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
        comm->writeData(endCommand, 4, 100000);
    #endif

}

void MainWindow::updateG(int value) {

    memset(buffer, 0x0, 100);

    switch(currentLed) {
        case 0: ledsColor[1] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 16, value);
                updateLed0();
                break;
        case 1: ledsColor[4] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 17, value);
                updateLed1();
                break;
        case 2: ledsColor[7] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 18, value);
                updateLed2();
                break;
        case 3: ledsColor[10] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 19, value);
                updateLed3();
                break;
        case 4: ledsColor[13] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 20, value);
                updateLed4();
                break;
        case 5: ledsColor[16] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 21, value);
                updateLed5();
                break;
        case 6: ledsColor[19] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 22, value);
                updateLed6();
                break;
        case 7: ledsColor[22] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 23, value);
                updateLed7();
                break;
        case 8: ledsColor[25] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 148, value);
                updateLed8();
                break;
    }

    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->WriteBuffer((BYTE*)endCommand, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
        comm->writeData(endCommand, 4, 100000);
    #endif

}

void MainWindow::updateB(int value) {

    memset(buffer, 0x0, 100);

    switch(currentLed) {
        case 0: ledsColor[2] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 8, value);
                updateLed0();
                break;
        case 1: ledsColor[5] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 9, value);
                updateLed1();
                break;
        case 2: ledsColor[8] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 10, value);
                updateLed2();
                break;
        case 3: ledsColor[11] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 11, value);
                updateLed3();
                break;
        case 4: ledsColor[14] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 12, value);
                updateLed4();
                break;
        case 5: ledsColor[17] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 13, value);
                updateLed5();
                break;
        case 6: ledsColor[20] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 14, value);
                updateLed6();
                break;
        case 7: ledsColor[23] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 15, value);
                updateLed7();
                break;
        case 8: ledsColor[26] = value;
                snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 147, value);
                updateLed8();
                break;
    }

    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->WriteBuffer((BYTE*)endCommand, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
        comm->writeData(endCommand, 4, 100000);
    #endif

}

void MainWindow::updateLed(int value) {
    currentLed = value;

    QObject::disconnect(ui->sliderR, SIGNAL(valueChanged(int)), this, SLOT(updateR(int)));
    QObject::disconnect(ui->sliderG, SIGNAL(valueChanged(int)), this, SLOT(updateG(int)));
    QObject::disconnect(ui->sliderB, SIGNAL(valueChanged(int)), this, SLOT(updateB(int)));
    ui->sliderR->setValue(ledsColor[currentLed*3]);
    ui->sliderG->setValue(ledsColor[currentLed*3+1]);
    ui->sliderB->setValue(ledsColor[currentLed*3+2]);
    QObject::connect(ui->sliderR, SIGNAL(valueChanged(int)), this, SLOT(updateR(int)));
    QObject::connect(ui->sliderG, SIGNAL(valueChanged(int)), this, SLOT(updateG(int)));
    QObject::connect(ui->sliderB, SIGNAL(valueChanged(int)), this, SLOT(updateB(int)));

}

void MainWindow::handleIR0(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 164, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR0();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 164, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR0();
    }
}

void MainWindow::handleIR1(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 165, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR1();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 165, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR1();
    }
}

void MainWindow::handleIR2(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 166, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR2();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 166, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR2();
    }
}

void MainWindow::handleIR3(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 167, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR3();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 167, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR3();
    }
}

void MainWindow::handleIR4(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 168, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR4();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 168, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR4();
    }
}

void MainWindow::handleIR5(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 169, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR5();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 169, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR5();
    }
}

void MainWindow::handleIR6(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 170, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR6();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 170, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR6();
    }
}

void MainWindow::handleIR7(int state) {
    if(state == Qt::Checked) {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 171, IRintensity);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOnIR7();
    } else {
        memset(buffer, 0x0, 100);
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 171, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->WriteBuffer((BYTE*)endCommand, 4);
            comm->FlushCommPort();
            Sleep(10);
            comm->PurgeCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
            comm->writeData(endCommand, 4, 100000);     // send the command to update the cycle
        #endif
        emit turnOffIR7();
    }
}

void MainWindow::updateIRIntensity(int value) {
    IRintensity = value;

    if(ui->chkIR0->isChecked()) {
        handleIR0(Qt::Checked);
    }
    if(ui->chkIR1->isChecked()) {
        handleIR1(Qt::Checked);
    }
    if(ui->chkIR2->isChecked()) {
        handleIR2(Qt::Checked);
    }
    if(ui->chkIR3->isChecked()) {
        handleIR3(Qt::Checked);
    }
    if(ui->chkIR4->isChecked()) {
        handleIR4(Qt::Checked);
    }
    if(ui->chkIR5->isChecked()) {
        handleIR5(Qt::Checked);
    }
    if(ui->chkIR6->isChecked()) {
        handleIR6(Qt::Checked);
    }
    if(ui->chkIR7->isChecked()) {
        handleIR7(Qt::Checked);
    }

}

void MainWindow::turnLedsOff() {
    int i=0;
    for(i=0; i<24; i++) {
        ledsColor[i] = 0;
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, i, 0);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->FlushCommPort();
        #else
            comm->writeData(buffer, 4, 100000);
        #endif
    }
    ledsColor[24] = 0;
    snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 146, 0);
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
    #endif
    ledsColor[25] = 0;
    snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 147, 0);
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
    #endif
    ledsColor[26] = 0;
    snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 148, 0);
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->WriteBuffer((BYTE*)endCommand, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
        comm->writeData(endCommand, 4, 100000);     // now the values are updates synchronously for all leds
    #endif

    updateLed0();
    updateLed1();
    updateLed2();
    updateLed3();
    updateLed4();
    updateLed5();
    updateLed6();
    updateLed7();
    updateLed8();

    QObject::disconnect(ui->sliderR, SIGNAL(valueChanged(int)), this, SLOT(updateR(int)));
    QObject::disconnect(ui->sliderG, SIGNAL(valueChanged(int)), this, SLOT(updateG(int)));
    QObject::disconnect(ui->sliderB, SIGNAL(valueChanged(int)), this, SLOT(updateB(int)));
    ui->sliderB->setValue(0);
    ui->sliderG->setValue(0);
    ui->sliderR->setValue(0);
    QObject::connect(ui->sliderR, SIGNAL(valueChanged(int)), this, SLOT(updateR(int)));
    QObject::connect(ui->sliderG, SIGNAL(valueChanged(int)), this, SLOT(updateG(int)));
    QObject::connect(ui->sliderB, SIGNAL(valueChanged(int)), this, SLOT(updateB(int)));

}

void MainWindow::turnLedsOn() {
    memset(buffer, 0x0, 100);
    int i=0;
    int bytes = 0;
    for(i=0; i<24; i++) {
        ledsColor[i] = 128;
        snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, i, 128);
        #ifdef __WIN32__
            comm->WriteBuffer((BYTE*)buffer, 4);
            comm->FlushCommPort();
        #else
            bytes = comm->writeData(buffer, 4, 100000);
        #endif
        std::cerr << "bytes written = " << bytes << std::endl;
        memset(buffer, 0x0, 100);
    }
    ledsColor[24] = 128;
    snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 146, 128);
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
    #endif
    ledsColor[25] = 128;
    snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 147, 128);
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
    #endif
    ledsColor[26] = 128;
    snprintf(buffer, 5, "%c%c%c%c",-'W', I2C_ADDRESS, 148, 128);
    #ifdef __WIN32__
        comm->WriteBuffer((BYTE*)buffer, 4);
        comm->WriteBuffer((BYTE*)endCommand, 4);
        comm->FlushCommPort();
    #else
        comm->writeData(buffer, 4, 100000);
        comm->writeData(endCommand, 4, 100000);     // now the values are updates synchronously for all leds
    #endif

    updateLed0();
    updateLed1();
    updateLed2();
    updateLed3();
    updateLed4();
    updateLed5();
    updateLed6();
    updateLed7();
    updateLed8();

    QObject::disconnect(ui->sliderR, SIGNAL(valueChanged(int)), this, SLOT(updateR(int)));
    QObject::disconnect(ui->sliderG, SIGNAL(valueChanged(int)), this, SLOT(updateG(int)));
    QObject::disconnect(ui->sliderB, SIGNAL(valueChanged(int)), this, SLOT(updateB(int)));
    ui->sliderB->setValue(128);
    ui->sliderG->setValue(128);
    ui->sliderR->setValue(128);
    QObject::connect(ui->sliderR, SIGNAL(valueChanged(int)), this, SLOT(updateR(int)));
    QObject::connect(ui->sliderG, SIGNAL(valueChanged(int)), this, SLOT(updateG(int)));
    QObject::connect(ui->sliderB, SIGNAL(valueChanged(int)), this, SLOT(updateB(int)));

}

