#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#define I2C_ADDRESS 176
#define MICRO_PAUSE 2000
#define ISR_TIMING 255000    // microseconds, found empirically

#include <QtGlobal>
#include <QtWidgets>
#include "ui_MainWindow.h"
#include <iostream>
#include "stdio.h"
#ifdef __WIN32__
        #include "comm.h"
#else
        #include "SerialComm.h"
#endif

namespace Ui
{
    class MainWindowClass;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    void init();
    ~MainWindow();

    Ui::MainWindowClass *ui;
    #ifdef __WIN32__
        TCommPort *comm;
    #else
        SerialComm *comm;
    #endif
    char buffer[100];                   // commands buffer
    char buffer2[100];                  // internal buffer (for coloring the leds)
    char endCommand[5];                 // end command (sent when the parameters must be updated)
    int currentLed;                     // current led to which apply modification
    int ledsColor[27];                  // max duty cycle (sequence is R0,G0,B0,R1,G1,B1,...R8,G8,B8)
    int IRintensity;                    // max duty cycle for IRs

    public slots:
        void connect();
        void updateR(int value);
        void updateG(int value);
        void updateB(int value);
        void updateLed(int value);
        void updateLed0();
        void updateLed1();
        void updateLed2();
        void updateLed3();
        void updateLed4();
        void updateLed5();
        void updateLed6();
        void updateLed7();
        void updateLed8();
        void turnOnIR0Slot();
        void turnOffIR0Slot();
        void turnOnIR1Slot();
        void turnOffIR1Slot();
        void turnOnIR2Slot();
        void turnOffIR2Slot();
        void turnOnIR3Slot();
        void turnOffIR3Slot();
        void turnOnIR4Slot();
        void turnOffIR4Slot();
        void turnOnIR5Slot();
        void turnOffIR5Slot();
        void turnOnIR6Slot();
        void turnOffIR6Slot();
        void turnOnIR7Slot();
        void turnOffIR7Slot();
        void handleIR0(int state);
        void handleIR1(int state);
        void handleIR2(int state);
        void handleIR3(int state);
        void handleIR4(int state);
        void handleIR5(int state);
        void handleIR6(int state);
        void handleIR7(int state);
        void updateIRIntensity(int value);
        void turnLedsOff();
        void turnLedsOn();

    signals:
        void turnOnIR0();
        void turnOffIR0();
        void turnOnIR1();
        void turnOffIR1();
        void turnOnIR2();
        void turnOffIR2();
        void turnOnIR3();
        void turnOffIR3();
        void turnOnIR4();
        void turnOffIR4();
        void turnOnIR5();
        void turnOffIR5();
        void turnOnIR6();
        void turnOffIR6();
        void turnOnIR7();
        void turnOffIR7();

};

#endif // MAINWINDOW_H
