/*
 *  SerialComm.cpp
 *  EPuckMonitor
 *
 *  Created by Stefano Morgani on 11/18/08.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#include "SerialComm.h"

SerialComm::SerialComm() {

}

SerialComm::~SerialComm() {

}

int SerialComm::connect(char *path, speed_t baud_rate) {
    int err = -2;

	// open the USB stream
        fd = open(path, O_RDWR | O_EXCL); // | O_NONBLOCK | O_NOCTTY | O_NDELAY);
        //fd = open(path, O_RDWR | O_NONBLOCK);
	if(fd==-1){
		return -1;
	}
	// get the baud rate and the flags
	struct termios term;
	tcgetattr(fd, &term);

        term.c_iflag = 0;
        term.c_oflag = 0;
        term.c_cflag = CLOCAL | CREAD | CS8;
        term.c_lflag = 0;
        //term.c_lflag &= ~(ICANON);		//non canonical reading
        //term.c_lflag |= ICANON;
        term.c_cc[ VTIME ] = 0; //1 timeout in tenths of a second
        term.c_cc[ VMIN ] = 1;  //0 Only wait for a single char

        //speed_t sp = cfgetispeed(&term);
	//fprintf(stderr,"The current speed is: %i\n", (int)sp);
        //sp = cfgetospeed(&term);
	//fprintf(stderr,"The current ospeed is: %i\n", (int)sp);
        err = cfsetispeed(&term, baud_rate);
        if(err != 0) {
            std::cerr << "error setting input speed" << std::endl;
        }
        err = cfsetospeed(&term, baud_rate);
        if(err != 0) {
            std::cerr << "error setting output speed" << std::endl;
        }
	//fprintf(stderr,"setting new baud rate ok: %i\n", cfsetispeed(&term, sp));
	//fprintf(stderr,"setting new baud rate ok: %i\n", cfsetospeed(&term, sp));
	//fprintf(stderr,"The current speed is: %i\n", (int)cfgetispeed(&term));
	//fprintf(stderr,"The current ospeed is: %i\n", (int)cfgetospeed(&term));
	
/*
	term.c_cflag |= (CLOCAL|CREAD|CS8);
	//because of O_NONBLOCK we have to set VTIME and VMIN
	term.c_lflag &= ~(ICANON);		//non canonical reading
	term.c_cc[VTIME] = 1;			//tenth of a second (1 => 100 ms)
	term.c_cc[VMIN] = 0;			//if no characters are in the buffer, the "read" return after VTIME tenth of a second	
*/

        if( tcsetattr(fd, TCSAFLUSH, &term) < 0)	//termios flags setting
	{
		perror("tcsetattr: ");
	}

	return 0;
}

void SerialComm::disconnect() {
	tcflush(fd,TCIOFLUSH);
	close(fd);
}

void SerialComm::flush() {
	tcflush(fd,TCIOFLUSH);
}

int SerialComm::simple_write(char *buf, int num_bytes) {
	//printf("Writing %s to serial port...\n", buf);
	int bytes_written = 0;	
	bytes_written = write(fd, buf, num_bytes);
	//tcflush(fd, TCOFLUSH);	
	return bytes_written;
}

int SerialComm::simple_read(char *buf, int num_bytes) {
	return read(fd, buf, num_bytes);
}

int SerialComm::writeData(char *buf, int num_bytes, int usleep_time) {
        //FILE *file = fopen("write.txt", "w");
	errno=0;
	//tcflush(fd,TCIOFLUSH);			//flush the input and output buffers
	int bytes_written=0;
	//int timeout = num_bytes*10;		//1 ms for every character
	int timeout = usleep_time/100;		//wait 100 useconds between every read; the loop lasts for usleep_time/100 times
	while(bytes_written<num_bytes) {
		bytes_written+=write(fd, &buf[bytes_written], num_bytes-bytes_written);
                if(errno != 0) {
                    perror("error writing: ");
                }
                //fprintf(file, "Error opening file: %s\n", strerror( errno ) );
		if(bytes_written<num_bytes) {
			//usleep(usleep_time);	
			usleep(100);				//1/(115200/8) = 70 useconds per byte
			timeout--;
		}
		if(timeout==0) {
			break;
		}
	}
        //fclose(file);
	return bytes_written;
}

int SerialComm::readData(char *buf, int num_bytes, int usleep_time) {
        //FILE *file = fopen("read.txt", "w");
        //errno=0;
	int bytes_red=0;
        int temp = 0;
	//int timeout = num_bytes*10;
	int timeout = usleep_time/100;		//wait 100 useconds between every read; the loop lasts for usleep_time/100 times
	while(bytes_red<num_bytes) {
                temp = read(fd,&buf[bytes_red],num_bytes-bytes_red);
                if(temp >= 0) {
                    bytes_red+=temp;
                }
		
                //fprintf(file, "Error opening file: %s\n", strerror( errno ) );

		if(bytes_red<num_bytes) {
			//usleep(usleep_time);
			usleep(100);				//1/(115200/8) = 70 useconds per byte
			timeout--;
		}
		if(timeout==0) {
                        printf("timeout in receiving...\n");
			break;
		}		
                //printf("bytes red = %d\n", bytes_red);
	}
	//tcflush(fd,TCIOFLUSH);			//flush the input and output buffers
        //fclose(file);
	return bytes_red;
}
